#!/usr/bin/env python

"""
Common resource functionality for Web calendar clients.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import datetime
from imiptools.client import Client, ClientForObject
from imiptools.data import get_uri, uri_values
from imiptools.dates import get_recurrence_start_point
from imiptools.period import remove_period, remove_affected_period
from imipweb.env import CGIEnvironment
import babel.dates
import imip_store
import markup

class Resource:

    "A Web application resource."

    def __init__(self, resource=None):

        """
        Initialise a resource, allowing it to share the environment of any given
        existing 'resource'.
        """

        self.encoding = "utf-8"
        self.env = CGIEnvironment(self.encoding)

        self.objects = {}
        self.locale = None
        self.requests = None

        self.out = resource and resource.out or self.env.get_output()
        self.page = resource and resource.page or markup.page()
        self.html_ids = None

    # Presentation methods.

    def new_page(self, title):
        self.page.init(title=title, charset=self.encoding, css=self.env.new_url("styles.css"))
        self.html_ids = set()

    def status(self, code, message):
        self.header("Status", "%s %s" % (code, message))

    def header(self, header, value):
        print >>self.out, "%s: %s" % (header, value)

    def no_user(self):
        self.status(403, "Forbidden")
        self.new_page(title="Forbidden")
        self.page.p("You are not logged in and thus cannot access scheduling requests.")

    def no_page(self):
        self.status(404, "Not Found")
        self.new_page(title="Not Found")
        self.page.p("No page is provided at the given address.")

    def redirect(self, url):
        self.status(302, "Redirect")
        self.header("Location", url)
        self.new_page(title="Redirect")
        self.page.p("Redirecting to: %s" % url)

    def link_to(self, uid, recurrenceid=None, section=None):

        """
        Return a link to an object with the given 'uid', 'recurrenceid' and
        'section'. See get_identifiers for the decoding of such links.
        """

        path = [uid]
        if recurrenceid:
            path.append(recurrenceid)
        if section:
            path.append(section)
        return self.env.new_url("/".join(path))

    # Control naming helpers.

    def element_identifier(self, name, index=None):
        return index is not None and "%s-%d" % (name, index) or name

    def element_name(self, name, suffix, index=None):
        return index is not None and "%s-%s" % (name, suffix) or name

    def element_enable(self, index=None):
        return index is not None and str(index) or "enable"

    # Access to objects.

    def get_identifiers(self, path_info):

        """
        Return identifiers provided by 'path_info', potentially encoded by
        'link_to'.
        """

        parts = path_info.lstrip("/").split("/")

        # UID only.

        if len(parts) == 1:
            return parts[0], None, None

        # UID and RECURRENCE-ID or UID and section.

        elif len(parts) == 2:
            if parts[1] == "counter":
                return parts[0], None, "counters"
            else:
                return parts[0], parts[1], parts[2] == "counter" and "counters" or None

        # UID, RECURRENCE-ID and section.

        else:
            return parts[:3]

    def _get_object(self, uid, recurrenceid=None, section=None):
        if self.objects.has_key((uid, recurrenceid, section)):
            return self.objects[(uid, recurrenceid, section)]

        obj = self.objects[(uid, recurrenceid, section)] = self.get_stored_object(uid, recurrenceid, section)
        return obj

    def _get_recurrences(self, uid):
        return self.store.get_recurrences(self.user, uid)

    def _get_active_recurrences(self, uid):
        return self.store.get_active_recurrences(self.user, uid)

    def _get_requests(self):
        if self.requests is None:
            self.requests = self.store.get_requests(self.user)
        return self.requests

    def _have_request(self, uid, recurrenceid=None, type=None, strict=False):
        return self.store.have_request(self._get_requests(), uid, recurrenceid, type, strict)

    def _get_request_summary(self):

        "Return a list of periods comprising the request summary."

        summary = []

        for uid, recurrenceid, request_type in self._get_requests():
            obj = self.get_stored_object(uid, recurrenceid)
            if obj:
                recurrenceids = self._get_active_recurrences(uid)

                # Obtain only active periods, not those replaced by redefined
                # recurrences, converting to free/busy periods.

                for p in obj.get_active_periods(recurrenceids, self.get_tzid(), self.get_window_end()):
                    summary.append(obj.get_freebusy_period(p))

        return summary

    # Preference methods.

    def get_user_locale(self):
        if not self.locale:
            self.locale = self.get_preferences().get("LANG", "en")
        return self.locale

    # Prettyprinting of dates and times.

    def format_date(self, dt, format):
        return self._format_datetime(babel.dates.format_date, dt, format)

    def format_time(self, dt, format):
        return self._format_datetime(babel.dates.format_time, dt, format)

    def format_datetime(self, dt, format):
        return self._format_datetime(
            isinstance(dt, datetime) and babel.dates.format_datetime or babel.dates.format_date,
            dt, format)

    def _format_datetime(self, fn, dt, format):
        return fn(dt, format=format, locale=self.get_user_locale())

    # Data management methods.

    def remove_request(self, uid, recurrenceid=None):
        return self.store.dequeue_request(self.user, uid, recurrenceid)

    def remove_event(self, uid, recurrenceid=None):
        return self.store.remove_event(self.user, uid, recurrenceid)

class ResourceClient(Resource, Client):

    "A Web application resource and calendar client."

    def __init__(self, resource=None):
        Resource.__init__(self, resource)
        user = self.env.get_user()
        Client.__init__(self, user and get_uri(user) or None)

class ResourceClientForObject(Resource, ClientForObject):

    "A Web application resource and calendar client for a specific object."

    def __init__(self, resource=None):
        Resource.__init__(self, resource)
        user = self.env.get_user()
        ClientForObject.__init__(self, None, user and get_uri(user) or None)

# vim: tabstop=4 expandtab shiftwidth=4
